@@ -14,7 +14,7 @@
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-/* Copyright (C) 2004-2023 Erik Auerswald <auerswal@unix-ag.uni-kl.de> */
+/* Copyright (C) 2004-2024 Erik Auerswald <auerswal@unix-ag.uni-kl.de> */
 /* Copyright (C) 2013 Cristiano Fontana <fontanacl@ornl.gov> */
 
 /* ImLib2 Header */
@@ -278,6 +278,7 @@ int main(int argc, char **argv)
   int dig_w;  /* width of digit part of image */
   int dig_h;  /* height of digit part of image */
   int max_dig_h=0, max_dig_w=0; /* maximum height & width of digits found */
+  int widest_dig_is_one=0; /* set to one if the widest digit is a one */
   Imlib_Color color; /* Imlib2 RGBA color structure */
   /* state of search */
   int state = (ssocr_foreground == SSOCR_BLACK) ? FIND_DARK : FIND_LIGHT;
@@ -379,7 +380,8 @@ int main(int argc, char **argv)
         if(optarg) {
           need_pixels = atoi(optarg);
           if(need_pixels < 1) {
-            fprintf(stderr, "warning: ignoring --number-pixels=%s\n", optarg);
+            fprintf(stderr, PROG ": warning: ignoring --number-pixels=%s\n",
+                    optarg);
             need_pixels = NEED_PIXELS;
           }
           if(flags & DEBUG_OUTPUT) {
@@ -391,7 +393,8 @@ int main(int argc, char **argv)
         if(optarg) {
           min_segment = atoi(optarg);
           if(min_segment < 1) {
-            fprintf(stderr, "warning: ignoring --min-segment=%s\n", optarg);
+            fprintf(stderr, PROG ": warning: ignoring --min-segment=%s\n",
+                    optarg);
             min_segment = MIN_SEGMENT;
             if(flags & DEBUG_OUTPUT) {
               fprintf(stderr, "min_segment = %d\n", min_segment);
@@ -409,7 +412,8 @@ int main(int argc, char **argv)
           int ret;
           ret = parse_width_height(optarg, &min_char_dims);
           if (ret) {
-            fprintf(stderr, "warning: ignoring --min-char-dims=%s\n", optarg);
+            fprintf(stderr, PROG ": warning: ignoring --min-char-dims=%s\n",
+                    optarg);
           }
           if(flags & DEBUG_OUTPUT) {
             fprintf(stderr, "min_char_dims = %dx%d\n", min_char_dims.w,
@@ -421,7 +425,8 @@ int main(int argc, char **argv)
         if(optarg) {
           ignore_pixels = atoi(optarg);
           if(ignore_pixels < 0) {
-            fprintf(stderr, "warning: ignoring --ignore-pixels=%s\n", optarg);
+            fprintf(stderr, PROG ": warning: ignoring --ignore-pixels=%s\n",
+                    optarg);
             ignore_pixels = IGNORE_PIXELS;
           }
         }
@@ -431,7 +436,8 @@ int main(int argc, char **argv)
           int ret;
           ret = parse_interval(optarg, &expected_digits);
           if(ret) {
-            fprintf(stderr, "warning: ignoring --number-digits=%s\n", optarg);
+            fprintf(stderr, PROG ": warning: ignoring --number-digits=%s\n",
+                    optarg);
             expected_digits.min = expected_digits.max = NUMBER_OF_DIGITS;
           }
           if (flags & DEBUG_OUTPUT) {
@@ -444,7 +450,7 @@ int main(int argc, char **argv)
         if(optarg) {
           one_ratio = atoi(optarg);
           if(one_ratio < 2) {
-            fprintf(stderr, "warning: ignoring --one-ratio=%s\n", optarg);
+            fprintf(stderr, PROG ": warning: ignoring --one-ratio=%s\n",optarg);
             one_ratio = ONE_RATIO;
           }
         }
@@ -453,7 +459,8 @@ int main(int argc, char **argv)
         if(optarg) {
           minus_ratio = atoi(optarg);
           if(minus_ratio < 1) {
-            fprintf(stderr, "warning: ignoring --minus-ratio=%s\n", optarg);
+            fprintf(stderr, PROG ": warning: ignoring --minus-ratio=%s\n",
+                    optarg);
             minus_ratio = MINUS_RATIO;
           }
         }
@@ -557,7 +564,8 @@ int main(int argc, char **argv)
         if(optarg) {
           dec_h_ratio = atoi(optarg);
           if(dec_h_ratio < 2) {
-            fprintf(stderr, "warning: ignoring --dec-h-ratio=%s\n", optarg);
+            fprintf(stderr, PROG ": warning: ignoring --dec-h-ratio=%s\n",
+                    optarg);
             dec_h_ratio = DEC_H_RATIO;
           }
         }
@@ -566,7 +574,8 @@ int main(int argc, char **argv)
         if(optarg) {
           dec_w_ratio = atoi(optarg);
           if(dec_w_ratio < 1) {
-            fprintf(stderr, "warning: ignoring --dec-w-ratio=%s\n", optarg);
+            fprintf(stderr, PROG ": warning: ignoring --dec-w-ratio=%s\n",
+                    optarg);
             dec_w_ratio = DEC_W_RATIO;
           }
         }
@@ -616,6 +625,7 @@ int main(int argc, char **argv)
   }
   if(flags & DEBUG_OUTPUT) {
     fprintf(stderr, "================================================================================\n");
+    fprintf(stderr, "VERSION=%s\n", VERSION);
     fprintf(stderr, "flags & VERBOSE=%d\nthresh=%f\n", flags & VERBOSE, thresh);
     fprintf(stderr, "flags & PRINT_INFO=%d\nflags & ADJUST_GRAY=%d\n",
             flags & PRINT_INFO, flags & ADJUST_GRAY);
@@ -623,7 +633,7 @@ int main(int argc, char **argv)
     fprintf(stderr, "flags & DO_ITERATIVE_THRESHOLD=%d\n",
                     flags & DO_ITERATIVE_THRESHOLD);
     fprintf(stderr, "flags & USE_DEBUG_IMAGE=%d\n", flags & USE_DEBUG_IMAGE);
-    fprintf(stderr, "flags & DEBUG_OUTPUT=%d\n", flags & PRINT_INFO);
+    fprintf(stderr, "flags & DEBUG_OUTPUT=%d\n", flags & DEBUG_OUTPUT);
     fprintf(stderr, "flags & PROCESS_ONLY=%d\n", flags & PROCESS_ONLY);
     fprintf(stderr, "flags & ASCII_ART_SEGMENTS=%d\n",
                     flags & ASCII_ART_SEGMENTS);
@@ -1352,8 +1362,15 @@ int main(int argc, char **argv)
   if ((expected_digits.min > -1) &&
       ((potential_digits < expected_digits.min) ||
        (potential_digits > expected_digits.max))) {
-    fprintf(stderr, PROG ": expected between %d and %d digits, but found %d\n",
-            expected_digits.min, expected_digits.max, potential_digits);
+    if (expected_digits.min != expected_digits.max) {
+      fprintf(stderr,
+              PROG ": expected between %d and %d digits, but found %d\n",
+              expected_digits.min, expected_digits.max, potential_digits);
+    } else {
+      fprintf(stderr, PROG ": expected %d digit%s, but found %d\n",
+              expected_digits.min, expected_digits.min > 1 ? "s" : "",
+              potential_digits);
+    }
     imlib_free_image_and_decache();
     if(flags & USE_DEBUG_IMAGE) {
       save_image("debug", debug_image, output_fmt,debug_image_file,flags);
@@ -1505,6 +1522,56 @@ int main(int argc, char **argv)
     }
   }
 
+  /* If the widest digit is a one, decimal points may be of the same width,
+   * and may thus not be detected.  Now that minus signs have been selected,
+   * if the widest digit still is a one (i.e., no minus signs), then decimal
+   * separators may also be recognized by checking only the height, not the
+   * width. */
+  if(flags & DEBUG_OUTPUT)
+    fputs("checking for special case of a one as widest character\n",stderr);
+  /* check if the widest digit is a one */
+  for(d=0; d<number_of_digits; d++) {
+    /* skip digits with zero width or height */
+    if((digits[d].x1 == digits[d].x2) || (digits[d].y1 == digits[d].y2)) {
+      if(flags & DEBUG_OUTPUT)
+        fprintf(stderr, " skipping digit %d with zero width or height\n", d);
+      continue;
+    }
+    if((digits[d].digit == D_ONE) && (digits[d].x2-digits[d].x1 >= max_dig_w)) {
+      widest_dig_is_one = 1;
+      if(flags & DEBUG_OUTPUT)
+        fputs(" widest digit is a one -> additional decimal point search\n",
+              stderr);
+      break;
+    }
+  }
+  if(!widest_dig_is_one) {
+    fputs(" widest digit is not a one, skipping extra decimal point search\n",
+          stderr);
+  } else {
+    /* widest digit is a one, thus decimal seperators may have been missed:
+     * identify a decimal point (or thousands separator) by relative height */
+    if(flags & DEBUG_OUTPUT)
+      fputs("looking for decimal points again\n",stderr);
+    for(d=0; d<number_of_digits; d++) {
+      /* skip digits with zero width or height */
+      if((digits[d].x1 == digits[d].x2) || (digits[d].y1 == digits[d].y2)) {
+        if(flags & DEBUG_OUTPUT)
+          fprintf(stderr, " skipping digit %d with zero width or height\n", d);
+        continue;
+      }
+      /* if height of a digit is less than 1/5 of the maximum digit height,
+       * and its width is less than 1/2 of the maximum digit width (the widest
+       * digit might be a one), assume it is a decimal point */
+      if((digits[d].digit == D_UNKNOWN) &&
+         (max_dig_h / (digits[d].y2 - digits[d].y1) > dec_h_ratio)) {
+        digits[d].digit = D_DECIMAL;
+        if(flags & DEBUG_OUTPUT)
+          fprintf(stderr, " digit %d is a decimal point\n", d);
+      }
+    }
+  }
+
   /* now the digits are located and they have to be identified */
   if(flags & DEBUG_OUTPUT)
     fputs("starting scanline based recognition for remaining digits\n", stderr);
